#ifndef DATUM_H
#define DATUM_H
#include <chrono>
#include <iostream>
#include <stdexcept>
#include <string>

class UngueltigesDatumException : public std::runtime_error {
public:
  UngueltigesDatumException(const std::string& str) 
  : std::runtime_error(str)
  { }
};

[[nodiscard]] constexpr bool istSchaltjahr(int jahr)
{
  return std::chrono::year(jahr).is_leap();
}

class [[nodiscard]] Datum {
public:
  Datum() =default;  // Standardkonstruktor

  constexpr Datum(int t, int m, int j) // allgemeiner Konstruktor
      : ymd(std::chrono::year(j), std::chrono::month(m), std::chrono::day(t))
  {
    if (!ymd.ok()) {
      throw UngueltigesDatumException(" Ungueltiges Datum!");
    }
  }

  [[nodiscard]] constexpr int tag() const
  {
    return static_cast<unsigned int>(ymd.day());
  }
  [[nodiscard]] constexpr int monat() const
  {
    return static_cast<unsigned int>(ymd.month());
  }
  [[nodiscard]] constexpr int jahr() const
  {
    return static_cast<int>(ymd.year());
  }

  Datum& operator++()                 // Tag hochzählen, Rückgabe per Referenz!
  {
    auto tage = std::chrono::sys_days(ymd);
    ymd = std::chrono::sys_days(tage + std::chrono::days{1});
    return *this;
  }
  
  Datum operator++(int) // Tag hochzählen, postfix
  {
    Datum temp{*this};
    ++*this; // Präfix ++ aufrufen
    return temp;
  }
  
  // alternativ: durch Methode toString() ersetzen (siehe
  // Übungsaufgabe)
  [[nodiscard]] explicit operator std::string() const //  Typumwandlungsoperator
  {
    return std::to_string(tag()) + "." + std::to_string(monat()) + "." +
           std::to_string(jahr());
  }

private:
  std::chrono::year_month_day ymd{
      std::chrono::floor<std::chrono::days>(std::chrono::system_clock::now())};
};

inline std::ostream& operator<<(std::ostream& os, const Datum& d)
{
  os << std::string(d);
  return os;
}

// Siehe auch Anmerkung zur Lösung 8.11 im Anhang
[[nodiscard]] constexpr bool operator==(const Datum& a, const Datum& b)
{
  return a.tag() == b.tag() && a.monat() == b.monat() && a.jahr() == b.jahr();
}

[[nodiscard]] constexpr bool operator!=(const Datum& a, const Datum& b)
{
  return !(a == b);
}

[[nodiscard]] constexpr bool operator<(const Datum& a, const Datum& b)
{
  return a.jahr() < b.jahr() ||
         (a.jahr() == b.jahr() && a.monat() < b.monat()) ||
         (a.jahr() == b.jahr() && a.monat() == b.monat() && a.tag() < b.tag());
}

[[nodiscard]] constexpr int datumDifferenz(const Datum& a, const Datum& b)
{
  if (a == b) // kurzer Prozeß bei Gleichheit
    return 0;
  bool richtigeReihenfolge{a < b};
  Datum frueher{a};
  Datum spaeter{b};
  if (!richtigeReihenfolge) { // ggf. vertauschen
    frueher = b;
    spaeter = a;
  }
  int differenz{0};
  // Achtung! nicht optimiert! (tageweises Hochzählen)
  while (frueher != spaeter) {
    ++differenz;
    ++frueher;
  }
  return richtigeReihenfolge ? differenz : -differenz;
}
#endif

