#ifndef CURLINTERFACE_H
#define CURLINTERFACE_H
#include <iostream>
#include <stdexcept>
#include <string>
#include <curl/curl.h>
// libcurl API documentation : https://curl.se/libcurl/c/libcurl.html

namespace {
constexpr bool debug {false}; 

// https://curl.se/libcurl/c/CURLOPT_WRITEFUNCTION.html
inline size_t writeFunction(char* ptr, size_t size, size_t nmemb,
                            std::string* data)
{
  data->append(ptr, size * nmemb);
  return size * nmemb;
}
}

class CurlInterface {
public:
  CurlInterface(const std::string& url)
  {
    curl_easy_setopt(curl, CURLOPT_URL, url.c_str());
    curl_easy_setopt(curl, CURLOPT_POST, 1);
    headers = curl_slist_append(headers, "Content-Type: application/json");
    const char* apiptr = std::getenv("OPENAI_API_KEY");
    if (!apiptr) {
      throw std::runtime_error(
          "Environment variable OPENAI_API_KEY not defined!");
    }
    std::string apikey {apiptr};
    const std::string authorization {"Authorization: Bearer " + apikey};
    headers = curl_slist_append(headers, authorization.c_str());
    curl_easy_setopt(curl, CURLOPT_HTTPHEADER, headers);
    curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, writeFunction);
    curl_easy_setopt(curl, CURLOPT_WRITEDATA, &response_str);
    curl_easy_setopt(curl, CURLOPT_SSL_VERIFYPEER, false);
        if (debug) {
      curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, error);
      curl_easy_setopt(curl, CURLOPT_VERBOSE, 1L);
    }
  }
  CurlInterface(const CurlInterface&) = delete;
  CurlInterface(CurlInterface&&) = delete;
  CurlInterface& operator=(const CurlInterface&) = delete;
  CurlInterface& operator=(CurlInterface&&) = delete;
  ~CurlInterface()
  {
    curl_slist_free_all(headers);
    curl_easy_cleanup(curl);
  }

  [[nodiscard]] auto getResponse(const std::string& request)
  {
    curl_easy_setopt(curl, CURLOPT_POSTFIELDS, request.c_str());
    CURLcode res = curl_easy_perform(curl);
    if (res != CURLE_OK) {
      std::string error_msg { curl_easy_strerror(res)};
      if (debug) {
        error_msg += error;
      }
      throw std::runtime_error(error_msg);
    }
    return response_str;
  }

private:
  CURL* curl {curl_easy_init()};
  struct curl_slist* headers {nullptr};
  std::string response_str {};
  char error[debug ? CURL_ERROR_SIZE : 1] {};
};
#endif
