/**
 * Diese Datei definiert keine systematischen Tests, sondern ist
 * ein Ansammlung von Beispielen für Testmakros. 
 */

#define BOOST_TEST_MODULE testbeispiele
#include <boost/test/included/unit_test.hpp>
#include <array>
#include <Datum.h>
#include <exception>
#include <fstream>          // Logging
#include <memory>           // unique_ptr
#include <vector>           // eq_collection

// Log-Datei definieren
// Level siehe /boost/test/detail/log_level.hpp
class LogKonfiguration {
public:
  LogKonfiguration() : test_log("test.log")
  {
    boost::unit_test::unit_test_log.set_stream(test_log);
    boost::unit_test::unit_test_log.set_threshold_level(boost::unit_test::log_test_units);
  }
  ~LogKonfiguration()          // schließen und zurücksetzen
  {
    test_log.close();
    boost::unit_test::unit_test_log.set_stream(std::cout);
  }
  LogKonfiguration(const LogKonfiguration&) = delete; 
  LogKonfiguration& operator=(const LogKonfiguration&) = delete; 
  
private:
  std::ofstream test_log;
};

// wird bei den Tests auf Werfen der Exception benutzt
inline bool checkException(const std::exception& e)
{
  return typeid(e) == typeid(UngueltigesDatumException);
}

BOOST_GLOBAL_FIXTURE(LogKonfiguration);

BOOST_AUTO_TEST_SUITE(testbeispiele)

BOOST_AUTO_TEST_CASE(konstruktor_mit_fixture)
{
  // unique_ptr<Datum> als Fixture
  std::unique_ptr<Datum> p = std::make_unique<Datum>(1, 1, 2024);     // Konstruktor (setUp)
  BOOST_CHECK(p->jahr() == 2024);
}          // delete automatisch durch unique_ptr-Destruktor

BOOST_AUTO_TEST_CASE(schaltjahrtest) { BOOST_CHECK_PREDICATE(istSchaltjahr, (2024)); }

BOOST_AUTO_TEST_CASE_EXPECTED_FAILURES(konstruktor_ungueltiges_Datum, 1)

BOOST_AUTO_TEST_CASE(konstruktor_ungueltiges_Datum) // soll fehlschlagen, kein Fehler:
{
  BOOST_CHECK_THROW(Datum(30, 2, 2023), UngueltigesDatumException);
  BOOST_TEST_MESSAGE("Gegenprobe: ");
  BOOST_CHECK_THROW(Datum(1, 2, 2023), UngueltigesDatumException);
}

BOOST_AUTO_TEST_CASE(konstruktor_exception)
{
  BOOST_CHECK_EXCEPTION(Datum(30, 2, 2024), UngueltigesDatumException, checkException);
}

BOOST_AUTO_TEST_CASE(konstruktor_nothrow)
{
  BOOST_CHECK_NO_THROW(do {
    int tag = 3;
    Datum(tag, 2, 2027);
  } while (0));
}

// Der folgende Testfall schlägt beabsichtigt fehl, damit
// BOOST_TEST_CHECKPOINT wirksam wird
BOOST_AUTO_TEST_CASE(konstruktor_ungueltiges_Datum_checkpoint)
{
  for (int tag = 1; tag < 30; ++tag) {
    BOOST_TEST_CHECKPOINT("Datum mit Tag = " << tag);
    Datum d(tag, 2, 2025);          // Exception bei tag == 29: Fehler
  }
}

BOOST_AUTO_TEST_CASE(differenz)
{
  Datum d(1, 1, 1800);
  Datum vortag;
  Datum ende(31, 12, 2300);
  while (d != ende) {
    vortag = d;
    ++d;
    BOOST_REQUIRE(vortag < d);          // operator<() testen
    BOOST_REQUIRE(datumDifferenz(vortag, d) == 11);
  }
}

BOOST_AUTO_TEST_CASE(relational) { BOOST_CHECK_EQUAL(17, 17); }

BOOST_AUTO_TEST_CASE(close)
{
  double toleranzInProzent = 2.0;
  BOOST_CHECK_CLOSE(1.25, 1.27, toleranzInProzent);
}

BOOST_AUTO_TEST_CASE(close_fraction)
{
  double toleranzAlsBruch = 0.016001;
  BOOST_CHECK_CLOSE_FRACTION(1.25, 1.27, toleranzAlsBruch);
}

BOOST_AUTO_TEST_CASE( eq_collection )
{
   std::vector<int> v;
   for (int i = 1; i <= 8; ++i) {
      v.push_back(i);
   }
   int arr [] = { 1, 2, 3, 5, 5, 6, 7, 9};
   BOOST_CHECK_EQUAL_COLLECTIONS(v.begin(), v.end(), arr, arr+8 );
}

BOOST_AUTO_TEST_CASE(eq_bit) { BOOST_CHECK_BITWISE_EQUAL(7, 9); }

BOOST_AUTO_TEST_SUITE_END()
